@@ -22,12 +22,8 @@ class SampleUpdate(SampleBase):
 
 # Properties shared by models stored in DB
 class SampleInDBBase(SampleBase):
-    id: int
     trace_id: int
 
-    class Config:
-        orm_mode = True
-
 
 # Properties to return to client
 class Sample(SampleInDBBase):
